&ANALYZE-SUSPEND _VERSION-NUMBER UIB_v9r12
&ANALYZE-RESUME
{adecomm/appserv.i}
DEFINE VARIABLE h_roundtable               AS HANDLE          NO-UNDO.
&ANALYZE-SUSPEND _UIB-CODE-BLOCK _CUSTOM _DEFINITIONS Procedure 
/*------------------------------------------------------------------------
File        : rtb_api.p
Description : API calls

About the API: 

This procedure provides a single point of access into Roundtable for
commonly-used, high-level functionality.

The internal procedures contained in this procedure are 'wrappers'.
These 'wrapper' procedures directly call the methods exposed in the
Roundtable SmartDataObjects. To guarantee compatibility in future
releases, we recommend you use this procedure as an entry point into
the Roundtable API.

However, the Roundtable SmartDataObjects can be dropped on an appropriate
Smart Container and their methods called directly.  Additionally, the
SmartDataObjects can also be instantiated using the 'fnRtbStartSuperSDO'
function as seen in the procedures contained herein.  Both of these
methods can be used to bypass the 'wrappers' if so desired.  Keep in mind
that compatibility cannot be guaranteed using this method as some of the
signatures may change in future releases.  Those changes will always be
reflected and documented in this procedure (rtb_api.p).  

Using the API:

Run this procedure persistently, obtaining a handle to access it's internal
procedures.

In order to use the routines in this procedure, you will first need to
login to the Roundtable repository.  This can be done by using the standard
Roundtable Login UI, or calling the 'login' internal procedure in this
procedure. 

If you are running the API stand-alone (no Roundtable session running), you
will need to supply both the session ID (obtained from the login process)
and the currently selected workspace to this procedure using the internal
procedures "set_session_id" and "set_workspace_id", respectively.  In addition,
you will need to run "set_paths" passing the PROPATH and your Roundtable
install directory.

  Ex:
    RUN login IN hApi (UserId,Password,OUTPUT sessionID,OUTPUT Error).
    RUN set_paths IN hApi ("c:/Roundtable",PROPATH,OUTPUT Error).
    RUN set_workspace_id IN hApi ("devel")

If the Roundtable Tabletop is active, this API procedure will inherit those
values automatically (see the Main Block) and the above steps are unnecessary.

Obtaining Additional Data:

Roundtable allows you to communicate with the Tabletop via published
events. Many values and handles are accessible in this manner. The
following events will return a value, when published (while the Tabletop
is active in the current session):

-- Usage Data --
evRtbGetCurrentTask (output INTEGER) - current task
evRtbGetCurrentUser (output CHAR) - current userid
evRtbGetCurrentView (output CHAR) - current tabletop view
evRtbGetCurrentWorkspace (output CHAR) - currently selected workspace
evRtbGetCurrentWsmod (output CHAR) - currently selected workspace module
evRtbGetSid (output INTEGER) - session ID

-- Record/Row Id's --
evRtbGetCurrentObjRowId (output ROWID ) - current rtb_object record
evRtbGetCurrentVersionRowid (output ROWID) - current rtb_ver record

-- Handles --
evRtbGetInUseSDOHandle (output HANDLE) - rtb_in-use SDO
evRtbGetModSDOHandle (output HANDLE) - rtb_module SDO
evRtbGetObjSDOHandle (output HANDLE) - rtb_object SDO
evRtbGetSessSDOHandle (output HANDLE) - rtb_session SDO
evRtbGetVerSDOHandle (output HANDLE) - rtb_ver SDO
evRtbGetWsSDOHandle (output HANDLE) - rtb_wspace SDO
evRtbGetDeskHandle (output HANDLE) - tabletop handle            

- Roundtable Development Team                    

------------------------------------------------------------------------*/

/* ***************************  Definitions  ************************** */
DEFINE VARIABLE Mwspace-id AS CHARACTER   NO-UNDO.
DEFINE VARIABLE Msid       AS INTEGER     NO-UNDO.


/* --- Temp-table needed by internal procedure create_xrefs --- */
DEFINE TEMP-TABLE TTxref
  FIELD src-object       AS CHARACTER
  FIELD ref-type         AS CHARACTER FORMAT "x(20)"
  FIELD obj-text         AS CHARACTER FORMAT "x(200)" EXTENT 4
  INDEX object AS PRIMARY
        src-object ASCENDING.

{rtb/i/rtb_super.i}

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME


&ANALYZE-SUSPEND _UIB-PREPROCESSOR-BLOCK 

/* ********************  Preprocessor Definitions  ******************** */

&Scoped-define PROCEDURE-TYPE Procedure
&Scoped-define DB-AWARE no



/* _UIB-PREPROCESSOR-BLOCK-END */
&ANALYZE-RESUME



/* *********************** Procedure Settings ************************ */

&ANALYZE-SUSPEND _PROCEDURE-SETTINGS
/* Settings for THIS-PROCEDURE
   Type: Procedure
   Allow: 
   Frames: 0
   Add Fields to: Neither
   Other Settings: CODE-ONLY COMPILE APPSERVER
 */
&ANALYZE-RESUME _END-PROCEDURE-SETTINGS

/* *************************  Create Window  ************************** */

&ANALYZE-SUSPEND _CREATE-WINDOW
/* DESIGN Window definition (used by the UIB) 
  CREATE WINDOW Procedure ASSIGN
         HEIGHT             = 20.76
         WIDTH              = 51.
/* END WINDOW DEFINITION */
                                                                        */
&ANALYZE-RESUME

 


&ANALYZE-SUSPEND _UIB-CODE-BLOCK _CUSTOM _MAIN-BLOCK Procedure 


/* ***************************  Main Block  *************************** */
/*
 Fire up standard function library..
*/
DYNAMIC-FUNCTION('fnRtbStartSuperProc':U,"rtb/p/rtb_funclib.p",SESSION).

/*
 Try to get the current session-id and workspace.  If Roundtable is
 already running, these values will be returned.  Otherwise, the user
 will need to run 'set_session_id' and 'set_workspace_id'.
*/
PUBLISH "evRtbGetSid":U (OUTPUT Msid).
PUBLISH "evRtbGetCurrentWorkspace":U (OUTPUT Mwspace-id).

/*
 Setup workspace environment if value was found..
*/
IF Mwspace-id <> "" THEN
  RUN set_workspace_id (INPUT Mwspace-id).

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME


/* **********************  Internal Procedures  *********************** */

&IF DEFINED(EXCLUDE-add_task) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE add_task Procedure 
PROCEDURE add_task :
/*------------------------------------------------------------------------------
  Purpose:     Create a Roundtable task
  Parameters:  Pwspace   - workspace to create task in (blank means current
                           wspace)
               Puser     - user id of task
               Pmanager  - manager of task
               Psummary  - task summary
               Puser-ref - user reference
               Pdesc     - task description
               Pshare    - share status (Central, Task, Group, Public)
               Ptask-dir - task directory
               Precid    - recid of task record
               Perror    - error message returned
  Notes:       
------------------------------------------------------------------------------*/
  DEFINE INPUT  PARAMETER Pwspace   AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER Puser     AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER Pmanager  AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER Psummary  AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER Puser-ref AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER Pdesc     AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER Pshare    AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER Ptask-dir AS CHARACTER NO-UNDO.
  DEFINE OUTPUT PARAMETER Precid    AS RECID     NO-UNDO.
  DEFINE OUTPUT PARAMETER Perror    AS CHARACTER NO-UNDO.

  DEFINE VARIABLE MhSdo     AS HANDLE      NO-UNDO.
  DEFINE VARIABLE Mtask-num AS INTEGER     NO-UNDO.
  DEFINE VARIABLE Mok       AS LOGICAL     NO-UNDO.

  MhSdo = fnRtbStartSuperSdo("rtb_u_task.w",THIS-PROCEDURE,NO).
  
  IF VALID-HANDLE(MhSdo) THEN DO:
    RUN rtb_createTask IN MhSdo (INPUT Msid,
                                 INPUT Pwspace,
                                 INPUT Pmanager,
                                 INPUT Puser,
                                 INPUT Psummary,
                                 INPUT Puser-ref,
                                 INPUT Pdesc,
                                 INPUT Pshare,
                                 INPUT Ptask-dir,
                                 OUTPUT Mtask-num, 
                                 OUTPUT Perror).
    IF Perror = "" THEN
      Precid = DYNAMIC-FUNCTION('fnRtbGetTaskRecid':U,Mtask-num).
    RUN destroyObject IN MhSdo.
  END.
  ELSE
    Perror = "Unable to initialize rtb_u_task.w".

  RETURN.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-assign_object) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE assign_object Procedure 
PROCEDURE assign_object :
/*------------------------------------------------------------------------------
  Purpose:     Assign object to workspace
  Parameters:  Ppmod     - product module
               Pobj-type - object type (PCODE)
               Pobject   - object name
               Pversion  - version of object to assign to current workspace
               Ptask-num - task number
               Precid    - recid of object assigned
               Perror    - error message returned
  Notes:       Before using the API, set the active task using the 'set_task'
               API procedure.
------------------------------------------------------------------------------*/
  DEFINE INPUT  PARAMETER Ppmod     AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER Pobj-type AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER Pobject   AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER Pversion  AS INTEGER   NO-UNDO.
  DEFINE INPUT  PARAMETER Ptask-num AS INTEGER   NO-UNDO.
  DEFINE OUTPUT PARAMETER Precid    AS RECID     NO-UNDO.
  DEFINE OUTPUT PARAMETER Perror    AS CHARACTER NO-UNDO.

  DEFINE VARIABLE MhSdo AS HANDLE NO-UNDO.

  MhSdo = fnRtbStartSuperSdo("rtb_u_repo.w",THIS-PROCEDURE,NO).

  IF VALID-HANDLE(MhSdo) THEN DO:

    IF Ptask-num > 0 THEN
      RUN set_task (Ptask-num).

    RUN rtb_assignObject IN MhSdo (INPUT Msid,
                                   INPUT Mwspace-id,
                                   INPUT Pobj-type,
                                   INPUT Pobject,
                                   INPUT Ppmod,
                                   INPUT Pversion,
                                   INPUT "",
                                   INPUT 0, 
                                   OUTPUT Perror).
                                   
    RUN destroyObject IN MhSdo.

    IF Perror = "" THEN DO:

      MhSdo = fnRtbStartSuperSdo("rtb_q_obj.w",THIS-PROCEDURE,NO).

      IF VALID-HANDLE(MhSdo) THEN DO:
        Precid = DYNAMIC-FUNCTION('fnRtbGetObjectRecid':U IN MhSdo,
                                  INPUT Mwspace-id,
                                  Pobject,
                                  Pobj-type).
        RUN destroyObject IN MhSdo.
      END.
      ELSE
        Perror = "Unable to initialize rtb_q_obj.w.".
    END.

  END.
  ELSE
    Perror = "Unable to initialize rtb_u_repo.w.".

  RETURN.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-check_in_object) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE check_in_object Procedure 
PROCEDURE check_in_object :
/*------------------------------------------------------------------------------
  Purpose:     Check in an object
  Parameters:  Proot-ver - YES to store full source / NO for incremental
               Ppmod     - product module
               Pobj-type - object type (PCODE)
               Pobject   - object name
               Ptask-num - task number
               Perror    - error message returned
  Notes:       
------------------------------------------------------------------------------*/
  DEFINE INPUT  PARAMETER Proot-ver AS LOGICAL   NO-UNDO.
  DEFINE INPUT  PARAMETER Ppmod     AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER Pobj-type AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER Pobject   AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER Ptask-num AS INTEGER   NO-UNDO.
  DEFINE OUTPUT PARAMETER Perror    AS CHARACTER NO-UNDO.

  DEFINE VARIABLE MhSdo AS HANDLE NO-UNDO.

  MhSdo = fnRtbStartSuperSdo("rtb_u_repo.w",THIS-PROCEDURE,NO).

  IF VALID-HANDLE(MhSdo) THEN DO:

    RUN set_task (Ptask-num).

    RUN rtb_checkInObject IN MhSdo (INPUT Msid,
                                    INPUT Mwspace-id,
                                    INPUT Ppmod,
                                    INPUT Pobj-type,
                                    INPUT Pobject,
                                    INPUT Proot-ver,
                                    OUTPUT Perror).

    RUN destroyObject IN MhSdo.
  END.
  ELSE
    Perror = "Unable to initialize rtb_u_repo.w.".

  RETURN.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-check_object_integrity) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE check_object_integrity Procedure 
PROCEDURE check_object_integrity :
/*------------------------------------------------------------------------------
  Purpose:     Check the integrity of a workspace object
  Parameters:  Pwspace   - workspace
               Ppmod     - product module
               Pobject   - object
               Pobj-type - object type
               Perror    - errors
  Notes:  This procedure compares the repository version of the object
          assigned to the given workspace and compares it to the physical
          copy in the workspace.     
------------------------------------------------------------------------------*/
  DEFINE INPUT  PARAMETER Pwspace   AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER Ppmod     AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER Pobject   AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER Pobj-type AS CHARACTER NO-UNDO.
  DEFINE OUTPUT PARAMETER Perror    AS CHARACTER NO-UNDO.

  DEFINE VARIABLE MhSdo AS HANDLE NO-UNDO.

  MhSdo = fnRtbStartSuperSdo("rtb_u_repo.w",THIS-PROCEDURE,NO).

  IF VALID-HANDLE(MhSdo) THEN DO:
    Perror = DYNAMIC-FUNCTION('fnRtbCheckObjectIntegrity':U IN MhSdo,
                              INPUT Pwspace,
                              INPUT Ppmod,
                              INPUT Pobj-type,
                              INPUT Pobject).
    RUN destroyObject IN MhSdo.
  END.
  ELSE
    Perror = "Unable to initialize rtb_u_repo.w.".

  RETURN.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-check_out_object) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE check_out_object Procedure 
PROCEDURE check_out_object :
/*------------------------------------------------------------------------------
  Purpose:     Check out an object
  Parameters:  Plevel    - patch, revision, version
               Ppmod     - product module of object
               Pobj-type - object type (PCODE)
               Pobject   - object name
               Ptask-num - task number
               Precid    - recid of object
               Perror    - error message returned
  Notes:       
------------------------------------------------------------------------------*/
  DEFINE INPUT  PARAMETER Plevel    AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER Ppmod     AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER Pobj-type AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER Pobject   AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER Ptask-num AS INTEGER   NO-UNDO.
  DEFINE OUTPUT PARAMETER Precid    AS RECID     NO-UNDO. 
  DEFINE OUTPUT PARAMETER Perror    AS CHARACTER NO-UNDO.

  DEFINE VARIABLE MhSdo             AS HANDLE    NO-UNDO.
  DEFINE VARIABLE MhTaskSdo         AS HANDLE    NO-UNDO.
  DEFINE VARIABLE Morphan           AS LOGICAL   NO-UNDO.
  DEFINE VARIABLE cTaskShareStatus  AS CHARACTER NO-UNDO.
  DEFINE VARIABLE rObjectRowid      AS ROWID     NO-UNDO.
 
  MhSdo = fnRtbStartSuperSdo("rtb_u_repo.w",THIS-PROCEDURE,NO).

  IF VALID-HANDLE(MhSdo) THEN DO:

    RUN set_task (Ptask-num).

    RUN rtb_checkOutObject IN MhSdo (INPUT Msid,
                                     INPUT Mwspace-id,
                                     INPUT Ppmod,
                                     INPUT Pobj-type,
                                     INPUT Pobject,
                                     INPUT Plevel,
                                     INPUT Ptask-num,
                                     INPUT "",
                                     INPUT "",
                                     OUTPUT Morphan, 
                                     OUTPUT Perror).
    RUN destroyObject IN MhSdo.

    IF Perror = "" THEN DO:
      /* 
        start object sdo 
      */
      MhSdo = fnRtbStartSuperSdo("rtb_u_obj.w",THIS-PROCEDURE,NO).

      IF VALID-HANDLE(MhSdo) THEN DO:
        Precid = DYNAMIC-FUNCTION('fnRtbGetObjectRecid':U IN MhSdo, Mwspace-id, Pobject, Pobj-type).
      END.
      ELSE
        Perror = "Unable to initialize rtb_u_obj.w.".
      
      IF Perror = "" THEN DO: 
             
        /*
          start task sdo
        */  
        MhTaskSdo = fnRtbStartSuperSdo("rtb_u_task.w",THIS-PROCEDURE,NO).
        IF NOT VALID-HANDLE(MhTaskSdo) THEN         
          Perror = "Unable to initialize rtb_u_task.w.".
          
        ELSE DO:            
          /* 
            get the rowid of the object and the share-status of the task
          */
          rObjectRowid      = DYNAMIC-FUNCTION('fnRtbGetObjectRowid':U IN MhSdo, Mwspace-id,Pobj-type,Pobject,Ppmod). 
          cTaskShareStatus  = DYNAMIC-FUNCTION('fnRtbGetTaskShareStatus':U IN MhTaskSdo, Ptask-num). 
 
          /*
            If the object type is NOT schema, and the task share-status is NOT Central, then
            we need to set the object share-status to match the task share-status:
          */          
          IF NOT DYNAMIC-FUNCTION('fnRtbIsSchemaObject':U IN MhSdo, Pobj-type) 
            AND (cTaskShareStatus <> "Central":U ) THEN DO: 
 
            /* 
              update the object's share status to match the task, 
              retrieve the object contents and put them in the right spot   
            */
            Perror = DYNAMIC-FUNCTION('fnRtbSetObjectShareStatus':U IN MhSdo,Msid,rObjectRowid,cTaskShareStatus).
            
          END. /* IF NOT DYNAMIC-FUNCTION('fnRtbIsSchemaObject'... */
                      
        END. /* else do */
          
        RUN destroyObject IN MhTaskSdo.
        RUN destroyObject IN MhSdo.
      
      END. /* IF Perror = "" THEN DO */
      
    END.
    
  END. /* IF VALID-HANDLE(MhSdo) THEN */
  ELSE
    Perror = "Unable to initialize rtb_u_repo.w.".

  RETURN.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-compile_object) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE compile_object Procedure 
PROCEDURE compile_object :
/*------------------------------------------------------------------------------
  Purpose:     Compile an object
  Parameters:  Ppmod     - product module
               Pobj-type - object type (PCODE)
               Pobject   - object name
               Psave     - IGNORED
               Pxref     - full compile with xref?
               Pxlst     - produce xref and listing files?
               Pshowerr  - show errors? This should be set to NO.
               Perror    - non-blank on warning or error with the format
                              
                              message<CHR(1)>filename
                           
                           where filename is path to file containing error
                           listing

  Notes:       The Psave option will be ignored. The saving of resultant
               r-code is determined by the workspace, module, and object
               settings affecting the specified object.
               
               Then why is it there??? - JAL
------------------------------------------------------------------------------*/
  DEFINE INPUT  PARAMETER Ppmod     AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER Pobj-type AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER Pobject   AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER Psave     AS LOGICAL   NO-UNDO.
  DEFINE INPUT  PARAMETER Pxref     AS LOGICAL   NO-UNDO.
  DEFINE INPUT  PARAMETER Pxlst     AS LOGICAL   NO-UNDO.
  DEFINE INPUT  PARAMETER Pshowerr  AS LOGICAL   NO-UNDO.
  DEFINE OUTPUT PARAMETER Perror    AS CHARACTER NO-UNDO.

  DEFINE VARIABLE MhSdo AS HANDLE NO-UNDO.

  MhSdo = fnRtbStartSuperSdo("rtb_u_obj.w",THIS-PROCEDURE,NO).

  IF VALID-HANDLE(MhSdo) THEN DO:
    Perror = DYNAMIC-FUNCTION('fnRtbCompileObject':U IN MhSdo,
                              INPUT Msid,
                              INPUT Mwspace-id,
                              INPUT Pobj-type,
                              INPUT Pobject,
                              INPUT Ppmod,
                              INPUT Pxref,
                              INPUT Pxlst,
                              INPUT 0).
    RUN destroyObject IN MhSdo.
  END.
  ELSE
    Perror = "Unable to initialize rtb_u_obj.w.".

  RETURN.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-complete_task) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE complete_task Procedure 
PROCEDURE complete_task :
/*------------------------------------------------------------------------------
  Purpose:     Complete a task
  Parameters:  Ptask-num - task number to complete (task numbers are unique
                           across workspaces)
               Pignore   - ignore compile warnings
               Perror    - non-blank on error

  Notes:       User may also run 'fnRtbCheckTaskDependencies' and
               'fnRtbCheckTaskObjectOrphans' in the SDO below before the call
               to 'rtb_completeTask'. Since dependencies and orphans are
               warnings only, the procedure call herein will complete the task
               even if dependencies and/or orphans exist.
------------------------------------------------------------------------------*/
  DEFINE INPUT  PARAMETER Ptask-num AS INTEGER   NO-UNDO.
  DEFINE INPUT  PARAMETER Pignore   AS LOGICAL   NO-UNDO.
  DEFINE OUTPUT PARAMETER Perror    AS CHARACTER NO-UNDO.

  DEFINE VARIABLE cWorkspace AS CHARACTER NO-UNDO.
  DEFINE VARIABLE MhSdo      AS HANDLE    NO-UNDO.
  DEFINE VARIABLE hCompiler  AS HANDLE    NO-UNDO.

  MhSdo = fnRtbStartSuperSdo("rtb_u_task.w",THIS-PROCEDURE,NO).
  
  IF VALID-HANDLE(MhSdo) THEN DO:

    DYNAMIC-FUNCTION('fnRtbSelectTask' IN MhSdo, Ptask-num).

    cWorkspace = ENTRY(2,DYNAMIC-FUNCTION('colValues':U IN MhSdo,"wspace-id":U), CHR(1)).

    /*
     Compiles the task objects
    */
    hCompiler = fnRtbStartSuperProc("rtb/p/rtb_compiler.p", THIS-PROCEDURE).

    RUN rtb_selective_compile IN hCompiler
      (INPUT cWorkspace,
       INPUT Ptask-num,
       INPUT "",    /* module */
       INPUT "",    /* object group */
       INPUT "",    /* object type */
       INPUT "",    /* object */        
       INPUT "",    /* pass blank pmod for selective compile */
       INPUT YES,   /* compile with xref */
       INPUT NO,    /* save xref and listings */
       INPUT NO,    /* force compilation */
       INPUT YES,   /* task completion? */
       INPUT Pignore, /* ignore warnings */
       INPUT NO,    /* stop on error ? */
       INPUT NO ).  /* yes = r-code for dependents in task dir */
  
    Perror = RETURN-VALUE.

    fnRtbEndSuperProc(THIS-PROCEDURE, hCompiler, YES).

    IF Perror = "" THEN
      RUN rtb_completeTask IN MhSdo
        (INPUT Msid,
         INPUT Ptask-num,
         OUTPUT Perror).
    
    RUN destroyObject IN MhSdo.

  END.
  ELSE
    Perror = "Unable to initialize rtb_u_task.w.".

  RETURN.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-create_object) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE create_object Procedure 
PROCEDURE create_object :
/*------------------------------------------------------------------------------
  Purpose:     Create an object in workspace
  Parameters:  Pobject      - object name
               Pobj-type    - object type (PCODE)   
               Psub-type    - sub type for the object (ignored when creating a
                              variant)
               Ppmod        - product module
                              Specify a pmod different than the current to
                              create a custom variant (move the object).
               Pgroup       - group to create this object in (ignored when
                              creating a variant)
               Plevel       - group level (ignored when creating variant)
               Pdescription - object description (ignored when creating variant)
               Poptions     - If the Pobject parameter begins with the "@"
                              character (an alias), this routine will check for
                              part names in the Poption string. This should be
                              a comma delimited list of relative path name and
                              file name.(ex: "reports/rpt1.p")
               Pui-on       - IGNORED
               Ptask-num    - task number
               Precid       - recid of object created
               Perror       - error message returned
  Notes:       Variant creation is provided for backward compatibility. For
               new interface usage, please use the create_variant procedure.
------------------------------------------------------------------------------*/
  DEFINE INPUT  PARAMETER Pobject      AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER Pobj-type    AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER Psub-type    AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER Ppmod        AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER Pgroup       AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER Plevel       AS INTEGER   NO-UNDO.
  DEFINE INPUT  PARAMETER Pdescription AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER Poptions     AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER Ptask-num    AS INTEGER   NO-UNDO.
  DEFINE INPUT  PARAMETER Pui-on       AS LOGICAL   NO-UNDO.
  DEFINE OUTPUT PARAMETER Precid       AS RECID     NO-UNDO. 
  DEFINE OUTPUT PARAMETER Perror       AS CHARACTER NO-UNDO.

  DEFINE VARIABLE Mobj-pmod AS CHARACTER   NO-UNDO.
  DEFINE VARIABLE MhSdo     AS HANDLE      NO-UNDO.

  MhSdo = fnRtbStartSuperSdo("rtb_u_obj.w",THIS-PROCEDURE,NO).

  IF VALID-HANDLE(MhSdo) THEN DO:

    RUN set_task (Ptask-num).

    /*
     If object already exists and specified pmod is different,
     then create a variant.
    */
    DYNAMIC-FUNCTION('fnRtbSelectWorkspaceTypeObject':U IN MhSdo,
                     Mwspace-id,
                     Pobj-type,
                     Pobject).
                     
    IF DYNAMIC-FUNCTION('rowAvailable':U IN MhSdo,?) THEN DO:
    
      Mobj-pmod = ENTRY(2,DYNAMIC-FUNCTION('colValues':U IN MhSdo,'pmod':U),CHR(1)).

      IF Mobj-pmod <> Ppmod THEN DO:
        RUN create_variant
          (INPUT  Pobj-type,
           INPUT  Pobject,
           INPUT  Ppmod,
           INPUT  Ptask-num,
           INPUT  Poptions,
           INPUT  "":U,
           INPUT  Plevel,
           OUTPUT Perror).
        RETURN.
      END.
      
    END.

    RUN rtb_newObject IN MhSdo (INPUT Msid,
                                INPUT Mwspace-id,
                                INPUT Ptask-num,
                                INPUT Pobject,
                                INPUT Pobj-type,
                                INPUT Psub-type,
                                INPUT Ppmod,
                                INPUT Pgroup,
                                INPUT Plevel,
                                INPUT Pdescription,
                                INPUT "",
                                INPUT Poptions, 
                                OUTPUT Perror).
                                
    IF Perror = "" THEN
      Precid = DYNAMIC-FUNCTION('fnRtbGetObjectRecid':U IN MhSdo,
                                INPUT Mwspace-id,
                                Pobject,
                                Pobj-type).
    RUN destroyObject IN MhSdo.
  END.
  ELSE
    Perror = "Unable to initialize rtb_u_obj.w.".

  RETURN.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-create_variant) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE create_variant Procedure 
PROCEDURE create_variant :
/*------------------------------------------------------------------------------
  Purpose:     Create a custom variant for the specified object with the
               specified product module.
  Parameters:  Pobj-type    - object type
               Pobject      - object name
               Pnew-pmod    - product module for variant
               Ptask-num    - task number
               Poption      - if alias object, comma-delimited relative
                              pathnames for object parts
               Pupd-notes   - version note for new variant
               Plevel       - check-out level
               Perror       - non-blank on error
  Notes:       
------------------------------------------------------------------------------*/
  DEFINE INPUT  PARAMETER Pobj-type  AS CHARACTER   NO-UNDO.
  DEFINE INPUT  PARAMETER Pobject    AS CHARACTER   NO-UNDO.
  DEFINE INPUT  PARAMETER Pnew-pmod  AS CHARACTER   NO-UNDO.
  DEFINE INPUT  PARAMETER Ptask-num  AS INTEGER     NO-UNDO.
  DEFINE INPUT  PARAMETER Poption    AS CHARACTER   NO-UNDO.
  DEFINE INPUT  PARAMETER Pupd-notes AS CHARACTER   NO-UNDO.
  DEFINE INPUT  PARAMETER Plevel     AS CHARACTER   NO-UNDO.
  DEFINE OUTPUT PARAMETER Perror     AS CHARACTER   NO-UNDO.

  DEFINE VARIABLE MrowIdent  AS CHARACTER NO-UNDO.
  DEFINE VARIABLE MhSdo      AS HANDLE    NO-UNDO.
  DEFINE VARIABLE Mobj-rowid AS ROWID     NO-UNDO.

  MhSdo = fnRtbStartSuperSdo("rtb_u_obj.w",THIS-PROCEDURE,NO).

  DYNAMIC-FUNCTION('fnRtbSelectWorkspaceTypeObject':U IN MhSdo,
                   Mwspace-id,
                   Pobj-type,
                   Pobject).

  {rtb/admtwo/get rowIdent MrowIdent MhSdo}.

  Mobj-rowid = TO-ROWID(MrowIdent).                 

  IF Mobj-rowid <> ? THEN DO:
    RUN set_task (Ptask-num).
    RUN rtb_createObjectVariant IN  MhSdo
      (INPUT  Msid,
       INPUT  Ptask-num,
       INPUT  Mobj-rowid,
       INPUT  Pnew-pmod,
       INPUT  Poption,
       INPUT  Pupd-notes,
       INPUT  Plevel,
       OUTPUT Perror).
   END.
   ELSE
     Perror = SUBSTITUTE("&1 object '&2' not found in '&3'.",
                         CAPS(Pobj-type),
                         Pobject,
                         Mwspace-id).

  RUN destroyObject IN MhSdo.

  RETURN.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-create_xrefs) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE create_xrefs Procedure 
PROCEDURE create_xrefs :
/*------------------------------------------------------------------------------
  Purpose:     Create xref records for non-Progress 4GL objects
  Parameters:  Input

                TTxref:

                A temp-table with the structure defined at the top of this file    
                that contains a record for every object relation that you want     
                to create where                                                    

                  src-object    = Name of object in the selected workspace that
                                  contains the reference (e.g. if "program.java"
                                  references "java.class", this field should 
                                  contain "program.java").

                  ref-type      = Valid reference type as listed in the Progress
                                  Language Reference under the COMPILE statement
                                  following the XREF parameter (i.e. "RUN",
                                  "INCLUDE", "CREATE", etc.).

                  obj-text[1-4] = Valid object identifier as shown in the
                                  Progress  Language Reference under the COMPILE
                                  statement following the XREF parameter. Each
                                  element of the object identifier separated by
                                  a space should be placed in an extent of this
                                  field.

                                  Examples:

                                   ref-type    : INCLUDE
                                   obj-text[1] : include-file-name
                                   obj-text[2] : ""
                                   obj-text[3] : ""
                                   obj-text[4] : ""

                                   ref-type    : ACCESS
                                   obj-text[1] : database.table
                                   obj-text[2] : field
                                   obj-text[3] : ""
                                   obj-text[4] : ""

                                   ref-type    : FUNCTION
                                   obj-text[1] : function-name,return-type
                                   obj-text[2] : "" 
                                   obj-text[3] : "" 
                                   obj-text[4] : "" 

                Plogfile:

                Pathname for file where errors will be logged.

                Papend:

                TRUE to append xref data to existing xref data for the
                specified object(s).

               Output

                Perror: 

                Contains applicable error message.

  Notes:       Logfile specified in Plogfile will be created in session temp
               directory.
------------------------------------------------------------------------------*/
  DEFINE INPUT  PARAMETER TABLE    FOR TTxref.
  DEFINE INPUT  PARAMETER Plogfile AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER Pappend  AS LOGICAL   NO-UNDO.
  DEFINE OUTPUT PARAMETER Perror   AS CHARACTER NO-UNDO.

  DEFINE VARIABLE Mlogfile  AS CHARACTER   NO-UNDO.
  DEFINE VARIABLE MhBuffer  AS HANDLE      NO-UNDO.
  DEFINE VARIABLE MhSdo     AS HANDLE      NO-UNDO.
  DEFINE VARIABLE MhTTxrefs AS HANDLE      NO-UNDO.

  /*
   Convert static table to a dynamic one...
  */
  CREATE TEMP-TABLE MhTTxrefs.
  MhTTxrefs:CREATE-LIKE("TTxref":U,"object":U).
  MhTTxrefs:TEMP-TABLE-PREPARE("xrefs").
  MhBuffer = MhTTxrefs:DEFAULT-BUFFER-HANDLE.

  FOR EACH TTxref:
    MhBuffer:BUFFER-CREATE.
    MhBuffer:BUFFER-COPY(BUFFER TTxref:HANDLE).
  END.

  /*
   Pass dynamic table to function
  */
  MhSdo = fnRtbStartSuperSdo("rtb_u_xref.w",THIS-PROCEDURE,NO).

  IF VALID-HANDLE(MhSdo) THEN DO:
    Perror = DYNAMIC-FUNCTION('fnRtbCreateXrefsFromTable':U,
                               INPUT Msid,
                               INPUT Mwspace-id,
                               INPUT TABLE-HANDLE MhTTxrefs,
                               INPUT-OUTPUT Plogfile,
                               INPUT Pappend).
    RUN destroyObject IN MhSdo.
  END.

  RETURN.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-delete_object) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE delete_object Procedure 
PROCEDURE delete_object :
/*------------------------------------------------------------------------------
  Purpose:     Delete an object from workspace
  Parameters:  Ppmod     - product module
               Pobj-type - object type (PCODE)
               Pobject   - object name
               Ptask-num - can be 0 or task-number (task number is needed if
                           obj is WIP or task is required by workspace)
               Pparams   - comma delimited list of optional parameters
                           "No-Prompt" - do not revert to previous version
                           "No-Delete-Source" - do not remove source
  Notes:       
------------------------------------------------------------------------------*/
  DEFINE INPUT  PARAMETER Ppmod     AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER Pobj-type AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER Pobject   AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER Ptask-num AS INTEGER   NO-UNDO.
  DEFINE INPUT  PARAMETER Pparams   AS CHARACTER NO-UNDO.
  DEFINE OUTPUT PARAMETER Perror    AS CHARACTER NO-UNDO.

  DEFINE VARIABLE MhSdo AS HANDLE NO-UNDO.

  MhSdo = fnRtbStartSuperSdo("rtb_u_obj.w",THIS-PROCEDURE,NO).

  IF VALID-HANDLE(MhSdo) THEN DO:

    IF Ptask-num > 0 THEN
      RUN set_task (Ptask-num).

    RUN rtb_removeObject IN MhSdo (INPUT Msid,
                                   INPUT Mwspace-id,
                                   INPUT Ppmod,
                                   INPUT Pobj-type,
                                   INPUT Pobject,
                                   INPUT Ptask-num,
                                   INPUT NOT CAN-DO(Pparams,"No-Prompt":U),
                                   INPUT NOT CAN-DO(Pparams,"No-Delete-Source":U), 
                                   INPUT "":U, /* Process*/
                                   OUTPUT Perror).
    RUN destroyObject IN MhSdo.
  END.
  ELSE
    Perror = "Unable to initialize rtb_u_obj.w.".

  RETURN.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-extract_object) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE extract_object Procedure 
PROCEDURE extract_object :
/*------------------------------------------------------------------------------
  Purpose:     Extract source code from repo
  Parameters:  Ppmod     - product module of object
               Pobj-type - object type (PCODE)
               Pobject   - object name
               Pversion  - version of object to extract
               Proot-dir - comma separated list of the explicit path and file
                           name for each part to be written out
               Perror    - error message returned
  Notes:       
------------------------------------------------------------------------------*/
  DEFINE INPUT  PARAMETER Ppmod     AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER Pobj-type AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER Pobject   AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER Pversion  AS INTEGER   NO-UNDO.
  DEFINE INPUT  PARAMETER Proot-dir AS CHARACTER NO-UNDO.
  DEFINE OUTPUT PARAMETER Perror    AS CHARACTER NO-UNDO.

  DEFINE VARIABLE Mfilelist  AS CHARACTER   NO-UNDO.
  DEFINE VARIABLE MhSdo      AS HANDLE      NO-UNDO.
  DEFINE VARIABLE Mi         AS INTEGER     NO-UNDO.
  DEFINE VARIABLE Mver-rowid AS ROWID       NO-UNDO.

  MhSdo = fnRtbStartSuperSdo("rtb_q_ver.w",THIS-PROCEDURE,NO).

  IF VALID-HANDLE(MhSdo) THEN DO:

    Mver-rowid = DYNAMIC-FUNCTION('fnRtbGetVersionRowid':U IN MhSdo,
                                  INPUT Pobj-type,
                                  INPUT Pobject,
                                  INPUT Ppmod,
                                  INPUT Pversion).
                                  
    RUN destroyObject IN MhSdo.

    MhSdo = fnRtbStartSuperSdo("rtb_u_repo.w",THIS-PROCEDURE,NO).
    
    IF VALID-HANDLE(MhSdo) THEN DO:

      Perror = DYNAMIC-FUNCTION('fnRtbExtractVersionToTemp':U IN MhSdo,
                                INPUT  Msid,
                                INPUT  Mver-rowid,
                                INPUT  0,
                                INPUT  NO,
                                OUTPUT Mfilelist).

      RUN destroyObject IN MhSdo.

      IF Perror = "" THEN
        DO Mi = 1 TO NUM-ENTRIES(Mfilelist):
          IF ENTRY(Mi,Proot-dir) <> "" THEN DO:
            OS-COPY VALUE(ENTRY(Mi,Mfilelist)) VALUE(ENTRY(Mi,Proot-dir)).
            OS-DELETE VALUE(ENTRY(Mi,Mfilelist)).
          END.
          ELSE DO:
            Perror = SUBSTITUTE("Not enough filenames supplied for object &1'",
                                Pobject).
            LEAVE.
          END.
        END.
        
    END.
    ELSE
      Perror = "Unable to initialize rtb_u_repo.w.".
  END.
  ELSE
    Perror = "Unable to initialize rtb_q_ver.w.".

  RETURN.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-login) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE login Procedure 
PROCEDURE login :
/*------------------------------------------------------------------------------
  Purpose:    Login to Roundtable
  Parameters: Puser - userid
              Ppass - password
              Psid  - new session id
              Perror - errors returned
  Notes:      DLCPath and RTBPath are both stored in the rtb_session record.
              DLCPath *should* be the unadulterated PROPATH while RTBPath
              should be the PROPATH with the Roundtable path prefixed to it.
              However, in this case they will both be identical. 
------------------------------------------------------------------------------*/
  DEFINE INPUT  PARAMETER Puser  AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER Ppass  AS CHARACTER NO-UNDO.
  DEFINE OUTPUT PARAMETER Psid   AS INTEGER   NO-UNDO.
  DEFINE OUTPUT PARAMETER Perror AS CHARACTER NO-UNDO.

  DEFINE VARIABLE MhSdo  AS HANDLE NO-UNDO.

  MhSdo = fnRtbStartSuperSdo("rtb_u_sess.w",THIS-PROCEDURE,NO).

  IF VALID-HANDLE(MhSdo) THEN DO:
    Perror = DYNAMIC-FUNCTION('fnRtbLogin':U IN MhSdo,Puser,Ppass,OUTPUT Psid).
    RUN destroyObject IN MhSdo.
  END.
  ELSE
    Perror = "Unable to initialize rtb_u_sess.w".

  /*
   Set the session ID variable scoped to the entire procedure..
  */
  IF Perror = "" THEN
    RUN set_session_id (INPUT Psid).

  RETURN.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-logout) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE logout Procedure 
PROCEDURE logout :
/*------------------------------------------------------------------------------
  Purpose:    Logout of Roundtable 
  Parameters: Psid - session ID to logout
  Notes:       
------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER  Psid   AS INTEGER   NO-UNDO.
  DEFINE OUTPUT PARAMETER Perror AS CHARACTER NO-UNDO.

  DEFINE VARIABLE MhSdo  AS HANDLE    NO-UNDO.

  MhSdo = fnRtbStartSuperSdo("rtb_u_sess.w",THIS-PROCEDURE,NO).

  IF VALID-HANDLE(MhSdo) THEN DO:
    Perror = DYNAMIC-FUNCTION('fnRtbLogout':U,Psid).  
    RUN destroyObject IN MhSdo.
  END.
  ELSE
    Perror = "Unable to initialize rtb_u_sess.w".

  RETURN. 

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-selective_compile) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE selective_compile Procedure 
PROCEDURE selective_compile :
/*------------------------------------------------------------------------------
  Purpose:     Perform a selective compile
  Parameters:  Ptask-num    - task number to select objects from or 0 to ignore
                              tasks    
               Pmodule      - module to select objects from or MATCHES pattern           
               Pobj-group   - Object group to select objects from or MATCHES
                              pattern     
               Pobj-type    - Object types to select (PCODE, PFILE, etc.) or
                              MATCHES pattern                                                    
               Pobject      - Object to select or MATCHES pattern                        
               Pforce       - Logical YES to force compile of (current) objects          
               Plistings    - Logical YES to generate compile listings                   
               Pxref        - Logical YES to perform xref of selected object(s)
               PtaskRcode   - Logical YES to generate Task r-code for dependencies
                              of Task objects
               PlIgnoreWarn - Logical YES to ignore compiler warnings           
               Perror       - This will be non-blank if the process could not be
                              performed
  Notes:       Compile errors and warnings will be recorded in the selcomp.log
               file created in the rtb_temp directory of the Roundtable session
               temp directory.                                                          
------------------------------------------------------------------------------*/
  DEFINE INPUT  PARAMETER Ptask-num    AS INTEGER   NO-UNDO.
  DEFINE INPUT  PARAMETER Pmodule      AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER Pobj-group   AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER Pobj-type    AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER Pobject      AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER Pforce       AS LOGICAL   NO-UNDO.
  DEFINE INPUT  PARAMETER Plistings    AS LOGICAL   NO-UNDO.
  DEFINE INPUT  PARAMETER Pxref        AS LOGICAL   NO-UNDO.
  DEFINE INPUT  PARAMETER PtaskRcode   AS LOGICAL   NO-UNDO.
  DEFINE INPUT  PARAMETER PlIgnoreWarn AS LOGICAL NO-UNDO.
  DEFINE OUTPUT PARAMETER Perror       AS CHARACTER NO-UNDO.

  DEFINE VARIABLE MhSdo AS HANDLE NO-UNDO.

  MhSdo = fnRtbStartSuperSdo("rtb_u_obj.w",THIS-PROCEDURE,NO).

  IF VALID-HANDLE(MhSdo) THEN DO:

    IF Ptask-num > 0 THEN
      RUN set_task (Ptask-num).

    RUN selective_compile IN MhSdo
      (INPUT Msid,
       INPUT Mwspace-id,
       INPUT Ptask-num,
       INPUT Pmodule,
       INPUT Pobj-group,
       INPUT Pobj-type,
       INPUT Pobject,
       INPUT "":U,  /* pass blank pmod for selective compile */
       INPUT Pxref,
       INPUT Plistings,
       INPUT Pforce,
       INPUT NO,       /* stop on error */
       INPUT PtaskRcode,
       INPUT PlIgnoreWarn,
       OUTPUT Perror).

    RUN destroyObject IN MhSdo.

    IF NUM-ENTRIES(Perror,CHR(1)) > 1 THEN DO:
      IF ENTRY(1,Perror,CHR(1)) EQ "INFO":U THEN
        Perror = ENTRY(2,Perror,CHR(1)).
      ELSE
        Perror = SUBSTITUTE("Warnings/Errors encountered. See &1 for details.",
                            ENTRY(2,Perror,CHR(1))).
    END.
    ELSE IF Perror = "Cancel":U THEN
      Perror = "Selective compile cancelled.".

  END.
  ELSE
    Perror = "Unable to initialize rtb_u_obj.w.".

  RETURN.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-set_paths) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE set_paths Procedure 
PROCEDURE set_paths :
/*------------------------------------------------------------------------------
  Purpose:     Set appropriate rtb_session paths
  Parameters:  PrtbPath - path to roundtable installation directory
               PdlcPath - default PROPATH
               Perror   - error
  Notes:       
------------------------------------------------------------------------------*/
  DEFINE INPUT  PARAMETER PrtbPath  AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER PdlcPath  AS CHARACTER NO-UNDO.
  DEFINE OUTPUT PARAMETER Perror AS CHARACTER NO-UNDO.

  DEFINE VARIABLE MhSdo  AS HANDLE NO-UNDO.

  PrtbPath = PrtbPath + "," + PdlcPath.

  MhSdo = fnRtbStartSuperSdo("rtb_u_sess.w",THIS-PROCEDURE,NO).

  IF VALID-HANDLE(MhSdo) THEN DO:
    Perror = DYNAMIC-FUNCTION('fnRtbSetDlcpath':U IN MhSdo,Msid,PdlcPath).
    IF Perror = "" THEN
      Perror = DYNAMIC-FUNCTION('fnRtbSetRtbpath':U IN MhSdo,Msid,PrtbPath).
    RUN destroyObject IN MhSdo.
  END.
  ELSE
    Perror = "Unable to initialize rtb_u_sess.w".

  RETURN.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-set_session_id) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE set_session_id Procedure 
PROCEDURE set_session_id :
/*------------------------------------------------------------------------------
  Purpose:     Sets the session ID.
  Parameters:  Psid = current session ID.
  Notes:       
------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER Psid AS INTEGER    NO-UNDO.

  Msid = Psid.
  RETURN.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-set_task) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE set_task Procedure 
PROCEDURE set_task :
/*------------------------------------------------------------------------------
  Purpose:     Set task number for session
  Parameters:  Ptask-num - active task
  Notes:       
------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER Ptask-num AS INTEGER NO-UNDO.
  
  DEFINE VARIABLE MhSdo AS HANDLE NO-UNDO.
  
  MhSdo = fnRtbStartSuperSdo("rtb_u_sess.w",THIS-PROCEDURE,NO).

  IF VALID-HANDLE(MhSdo) THEN DO:
    DYNAMIC-FUNCTION('fnRtbSetSessionProperty':U IN MhSdo,"task":U,Ptask-num).
    RUN destroyObject IN MhSdo.
  END.
  
  RETURN.
  
END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-set_workspace_id) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE set_workspace_id Procedure 
PROCEDURE set_workspace_id :
/*------------------------------------------------------------------------------
  Purpose:     Set workspace ID for use in API routines and set the PROPATH
               accordingly.
  Parameters:  Pwspace-id = Workspace ID of selected workspace.
  Notes:       
------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER Pwspace-id AS CHARACTER   NO-UNDO.

  Mwspace-id = Pwspace-id.

  IF NOT SESSION:REMOTE THEN
    DYNAMIC-FUNCTION('fnRtbSetPropath':U,Msid,Mwspace-id,0).

  RETURN.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-unlock_object) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE unlock_object Procedure 
PROCEDURE unlock_object :
/*------------------------------------------------------------------------------
  Purpose:     Unlock an object
  Parameters:  Pwspace   - workspace
               Ppmod     - product module
               Pobject   - object
               Pobj-type - object type
               Perror    - error
  Notes:       
------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER Pwspace   AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER Ppmod     AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER Pobject   AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER Pobj-type AS CHARACTER NO-UNDO.

  DEFINE OUTPUT PARAMETER Perror AS CHARACTER NO-UNDO.

  DEFINE VARIABLE Mpartnames AS CHARACTER   NO-UNDO.
  DEFINE VARIABLE Mvalues    AS CHARACTER   NO-UNDO.
  DEFINE VARIABLE MhSdo      AS HANDLE      NO-UNDO.
  DEFINE VARIABLE Mtask-num  AS INTEGER     NO-UNDO.
  DEFINE VARIABLE Mver-rowid AS ROWID       NO-UNDO.

  MhSdo = fnRtbStartSuperSdo("rtb_q_obj.w",THIS-PROCEDURE,NO).

  IF VALID-HANDLE(MhSdo) THEN DO:
  
    DYNAMIC-FUNCTION('fnRtbSelectWorkspaceTypeObject':U,
                     INPUT Pwspace,
                     INPUT Pobj-type,
                     INPUT Pobject).
    
    Mvalues = DYNAMIC-FUNCTION('colValues':U IN MhSdo,"objTask,objVersionRowid":U).
    
    ASSIGN Mtask-num  = INTEGER(ENTRY(2,Mvalues,CHR(1)))
           Mver-rowid = TO-ROWID(ENTRY(3,Mvalues,CHR(1))).
    
    RUN destroyObject IN MhSdo.
    
    IF Mver-rowid <> ? THEN DO:

      MhSdo = fnRtbStartSuperSdo("rtb_q_ver.w",THIS-PROCEDURE,NO).

      IF VALID-HANDLE(MhSdo) THEN DO:
        Mpartnames = DYNAMIC-FUNCTION('fnRtbGetVersionEditFilenames':U IN MhSdo,
                                      INPUT  Msid,
                                      INPUT  Pwspace,
                                      INPUT  Mtask-num,
                                      INPUT  Mver-rowid,
                                      INPUT  NO,
                                      OUTPUT Perror).

        RUN destroyObject IN MhSdo.

        IF Perror = "" THEN DO:

          MhSdo = fnRtbStartSuperSdo("rtb_u_inuse.w",THIS-PROCEDURE,NO).

          IF VALID-HANDLE(MhSdo) THEN DO:
            Perror = DYNAMIC-FUNCTION('fnRtbUnlockObject':U IN MhSdo,
                                      INPUT Msid,
                                      INPUT Pwspace,
                                      INPUT Ppmod,
                                      INPUT Pobj-type,
                                      INPUT Pobject,
                                      INPUT Mpartnames).
            RUN destroyObject IN MhSdo.
          END.
          ELSE
            Perror = "Unable to initialize rtb_u_inuse.w.".
        END. /* perror = "" */
      END.
      ELSE
        Perror = "Unable to initialize rtb_q_ver.w.".
    END.  /* mver-rowid <> ? */
    ELSE
      Perror = "Specified object record was not found.".
  END.
  ELSE
    Perror = "Unable to initialize rtb_u_obj.w.".

  RETURN.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

