/*------------------------------------------------------------------------
  File: rtb_runbuildprog.p

  Description: Runs specified PCODE Subtype build program.

  Input Parameters:
      Pbuild-prog  - Pathname of build program.
      PhTTobj-data - Temp-table containing object data.

  Output Parameters:
      <none>

  Notes:   This procedure will run a user build program in one of two
           ways, depending upon the value of the LEGACY preprocessor:
           
           &SCOPED-DEFINE LEGACY 0:
           
             RUN VALUE(Pbuild-prog)
               (INPUT <object-name> AS CHARACTER,
                INPUT <comma-delimted-parts-list> AS CHARACTER,
                INPUT <comma-delimited-part-descriptions> AS CHARACTER,
                INPUT <comma-delimited-part-pathnames> AS CHARACTER,
                INPUT <comma-delimited-part-names> /* no path */ AS CHARACTER,
                INPUT <comma-delimited-part-templates> AS CHARACTER,
                INPUT <number-of-parts> AS INTEGER,
                INPUT <code-subtype> AS CHARACTER,
                INPUT <user-id> AS CHARACTER,
                INPUT <task-num> AS INTEGER,
                INPUT <task-summary> AS CHARACTER,
                INPUT <rtb-propath> AS CHARACTER,
                INPUT <workspace-path> AS CHARACTER,
                INPUT <workspace-module-direcory> AS CHARACTER).
           
           &SCOPED-DEFINE LEGACY 1:
           
              RUN VALUE(Pbuild-prog).
            
              with NEW SHARED Urtb-* variables defined.
              
           The temp-table passed to this procedure contains the following
           fields:
           
              "object"     - The object name.
              "parts"      - Comma-delimited list of part identifiers.
              "part-descs" - Comma-delimited list of part descriptions.
              "part-paths" - Comma-delimited list of full pathnames to parts.
              "part-names" - Comma-delimited list of part names (no paths).
              "part-tmpls" - Comma-delimited list of part templates.
              "num-parts"  - Number of parts for specified object.
              "sub-type"   - Code subtype of specified part.
              "user-id"    - User ID of calling session.
              "task-num"   - Selected task number.
              "task-summ"  - Summary of specified task.
              "rtb-path"   - Roundtable startup propath.
              "ws-path"    - Selected Workspace path.
              "mod-dir"    - Workspace module source directory.
              
           The specified build program will run on both the server
           and client partitions when running the Roundtable GUI client
           in AppServer mode. Otherwise, the build program will fire
           on the server.
------------------------------------------------------------------------*/ 
DEFINE INPUT PARAMETER Pbuild-prog AS CHARACTER NO-UNDO.
DEFINE INPUT PARAMETER TABLE-HANDLE PhTTobjdata.

/*
 If using legacy build programs that reference Urtb-* variables, please
 set the LEGACY preprocesser value below to 1
*/
&SCOPED-DEFINE LEGACY 0

&IF {&LEGACY} &THEN 
  DEFINE NEW SHARED VARIABLE Urtb-object     AS CHARACTER           NO-UNDO.
  DEFINE NEW SHARED VARIABLE Urtb-part       AS CHARACTER EXTENT 10 NO-UNDO.
  DEFINE NEW SHARED VARIABLE Urtb-part-desc  AS CHARACTER EXTENT 10 NO-UNDO.
  DEFINE NEW SHARED VARIABLE Urtb-path       AS CHARACTER EXTENT 10 NO-UNDO.
  DEFINE NEW SHARED VARIABLE Urtb-name       AS CHARACTER EXTENT 10 NO-UNDO.
  DEFINE NEW SHARED VARIABLE Urtb-tmpl       AS CHARACTER EXTENT 10 NO-UNDO.
  DEFINE NEW SHARED VARIABLE Urtb-num-parts  AS INTEGER             NO-UNDO.
  DEFINE NEW SHARED VARIABLE Urtb-sub-type   AS CHARACTER           NO-UNDO.
  DEFINE NEW SHARED VARIABLE Urtb-userid     AS CHARACTER           NO-UNDO.
  DEFINE NEW SHARED VARIABLE Urtb-task-num   AS INTEGER             NO-UNDO.
  DEFINE NEW SHARED VARIABLE Urtb-task-desc  AS CHARACTER           NO-UNDO.
  DEFINE NEW SHARED VARIABLE Urtb-propath    AS CHARACTER           NO-UNDO.
  DEFINE NEW SHARED VARIABLE Urtb-ws-propath AS CHARACTER           NO-UNDO.
  DEFINE NEW SHARED VARIABLE Urtb-mod-dir    AS CHARACTER           NO-UNDO.
&ELSE
  DEFINE VARIABLE Mobject    AS CHARACTER   NO-UNDO.
  DEFINE VARIABLE Mparts     AS CHARACTER   NO-UNDO.
  DEFINE VARIABLE MpartDescs AS CHARACTER   NO-UNDO.
  DEFINE VARIABLE MpartPaths AS CHARACTER   NO-UNDO.
  DEFINE VARIABLE MpartNames AS CHARACTER   NO-UNDO.
  DEFINE VARIABLE MpartTmpls AS CHARACTER   NO-UNDO.
  DEFINE VARIABLE MnumParts  AS INTEGER     NO-UNDO.
  DEFINE VARIABLE MsubType   AS CHARACTER   NO-UNDO.
  DEFINE VARIABLE MuserId    AS CHARACTER   NO-UNDO.
  DEFINE VARIABLE MtaskNum   AS INTEGER     NO-UNDO.
  DEFINE VARIABLE MtaskSumm  AS CHARACTER   NO-UNDO.
  DEFINE VARIABLE Mpropath   AS CHARACTER   NO-UNDO.
  DEFINE VARIABLE MwsPath    AS CHARACTER   NO-UNDO.
  DEFINE VARIABLE MmodDir    AS CHARACTER   NO-UNDO.
&ENDIF

DEFINE VARIABLE MhBuffer AS HANDLE      NO-UNDO.
DEFINE VARIABLE Mi       AS INTEGER     NO-UNDO.

/*
 Set user variables from supplied object data
*/
MhBuffer = PhTTobjdata:DEFAULT-BUFFER-HANDLE.
MhBuffer:FIND-FIRST().

IF MhBuffer:AVAILABLE THEN DO:
  
  &IF {&LEGACY} &THEN
    
    ASSIGN Urtb-object      = MhBuffer:BUFFER-FIELD("object":U):BUFFER-VALUE
           Urtb-num-parts   = MhBuffer:BUFFER-FIELD("num-parts":U):BUFFER-VALUE
           Urtb-sub-type    = MhBuffer:BUFFER-FIELD("sub-type":U):BUFFER-VALUE
           Urtb-userid      = MhBuffer:BUFFER-FIELD("user-id":U):BUFFER-VALUE
           Urtb-task-num    = MhBuffer:BUFFER-FIELD("task-num":U):BUFFER-VALUE
           Urtb-task-desc   = MhBuffer:BUFFER-FIELD("task-summ":U):BUFFER-VALUE
           Urtb-propath     = MhBuffer:BUFFER-FIELD("rtb-path":U):BUFFER-VALUE
           Urtb-ws-propath  = MhBuffer:BUFFER-FIELD("ws-path":U):BUFFER-VALUE
           Urtb-mod-dir     = MhBuffer:BUFFER-FIELD("mod-dir":U):BUFFER-VALUE.
           
    DO Mi = 1 TO NUM-ENTRIES(MhBuffer:BUFFER-FIELD("parts":U):BUFFER-VALUE):
      Urtb-part[Mi]      = ENTRY(Mi,MhBuffer:BUFFER-FIELD("parts":U):BUFFER-VALUE).
      Urtb-part-desc[Mi] = ENTRY(Mi,MhBuffer:BUFFER-FIELD("part-descs":U):BUFFER-VALUE).
      Urtb-path[Mi]      = ENTRY(Mi,MhBuffer:BUFFER-FIELD("part-paths":U):BUFFER-VALUE).
      Urtb-name[Mi]      = ENTRY(Mi,MhBuffer:BUFFER-FIELD("part-names":U):BUFFER-VALUE).
      Urtb-tmpl[Mi]      = ENTRY(Mi,MhBuffer:BUFFER-FIELD("part-tmpls":U):BUFFER-VALUE).
    END.
    
  &ELSE
    ASSIGN Mobject    = MhBuffer:BUFFER-FIELD("object":U):BUFFER-VALUE
           Mparts     = MhBuffer:BUFFER-FIELD("parts":U):BUFFER-VALUE
           MpartDescs = MhBuffer:BUFFER-FIELD("part-descs":U):BUFFER-VALUE
           MpartPaths = MhBuffer:BUFFER-FIELD("part-paths":U):BUFFER-VALUE
           MpartNames = MhBuffer:BUFFER-FIELD("part-names":U):BUFFER-VALUE
           MpartTmpls = MhBuffer:BUFFER-FIELD("part-tmpls":U):BUFFER-VALUE
           MnumParts  = MhBuffer:BUFFER-FIELD("num-parts":U):BUFFER-VALUE
           MsubType   = MhBuffer:BUFFER-FIELD("sub-type":U):BUFFER-VALUE
           MuserId    = MhBuffer:BUFFER-FIELD("user-id":U):BUFFER-VALUE
           MtaskNum   = MhBuffer:BUFFER-FIELD("task-num":U):BUFFER-VALUE
           MtaskSumm  = MhBuffer:BUFFER-FIELD("task-summ":U):BUFFER-VALUE
           Mpropath   = MhBuffer:BUFFER-FIELD("rtb-path":U):BUFFER-VALUE
           MwsPath    = MhBuffer:BUFFER-FIELD("ws-path":U):BUFFER-VALUE
           MmodDir    = MhBuffer:BUFFER-FIELD("mod-dir":U):BUFFER-VALUE.
  &ENDIF
  
END.

/*
 Run the build program, if it exists
*/
IF SEARCH(Pbuild-prog) <> ? THEN DO:
  
  RUN VALUE(Pbuild-prog)
    &IF {&LEGACY} &THEN
      .
    &ELSE
      (INPUT Mobject,
       INPUT Mparts,
       INPUT MpartDescs,
       INPUT MpartPaths,
       INPUT MpartNames,
       INPUT MpartTmpls,
       INPUT MnumParts,
       INPUT MsubType,
       INPUT MuserId,
       INPUT MtaskNum,
       INPUT MtaskSumm,
       INPUT Mpropath,
       INPUT MwsPath,
       INPUT MmodDir).
    &ENDIF
  
END.
ELSE
  /*
   This should write to the log if running on the appserver. I am not raising the
   ERROR status because the missing build program should not stop the whole process.
  */
  MESSAGE SUBSTITUTE("Build program '&1' for Subtype '&2' was not found.",
    Pbuild-prog, &IF {&LEGACY} &THEN Urtb-sub-type &ELSE MsubType &ENDIF)
    VIEW-AS ALERT-BOX ERROR TITLE "Roundtable".
    
DELETE OBJECT PhTTobjdata NO-ERROR.

RETURN.
